import java.io.Reader;
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.io.StreamTokenizer;
import java.util.Iterator;
import java.util.Set;
import java.util.List;
import java.util.LinkedList;

/* The "main" class */
public class Quine
{
  static int numVars;
  static boolean min;
  static List<Integer> terms;
  static List<Integer> dontCares;

  /* The "main" method */
  static public void main(String args[])
  {
    /* Allocate objects for the input terms and don't cares; parse the input */
    terms = new LinkedList<Integer>();
    dontCares = new LinkedList<Integer>();

    parseInput();

    /* Merge the terms and don't cares into a single list */
    List<Integer> termsPlusDontCares = new LinkedList<Integer>();
    termsPlusDontCares.addAll(terms);
    termsPlusDontCares.addAll(dontCares);

    /* Build a new implication table, and find all the prime implicants */
    ImplicationTable t = new ImplicationTable(numVars, termsPlusDontCares);
    Set<Implicant> primes = t.getPrimeImplicants();

    for (Implicant imp : primes) {
      // System.out.println("prime: "+imp);
    }
    
    /* Build a new prime implicant chart and find a valid cover of the input
     * function */
    PrimeImplicantChart chart = new PrimeImplicantChart(terms, primes);
    Set<Implicant> cover = chart.minimumCover();

    for (Implicant imp : cover) {
      //System.out.println("cover: "+imp);
    }

    /* Print the resulting cover */
    printExpression(cover);
  }

  static private void printExpression(Set<Implicant> imps)
  {
    //System.out.println("Expression: ");
    boolean first = true;
    for (Implicant imp : imps) {
      if (!first) {
        if (min)
          System.out.print("+");
      }
      first = false;

      System.out.print("(");
      char v = 'A';
      boolean firstVar = true;
      for (Bit b : imp.bits()) {
        String op = "";
        if ((!min) && (!firstVar)) {
          op = "+";
        }
        if (b instanceof BitOne)
          System.out.print(op+v);
        if (b instanceof BitZero)
          System.out.print(op+"~"+v);
        if (!(b instanceof BitBoth))
          firstVar = false;
        v++;
      }
      System.out.print(")");
    }
    System.out.println("");
  }

  static private void parseInput()
  {
    Reader r = new BufferedReader(new InputStreamReader(System.in));
    StreamTokenizer st = new StreamTokenizer(r);

    int ttype;
    try {
      ttype = st.nextToken();
      if (ttype == StreamTokenizer.TT_NUMBER) {
        numVars = (int)st.nval;
        if ((numVars >= 26) || (numVars < 0)) {
          System.err.println("Q-M 370 can only handle up to 26 varaibles");
          System.exit(1);
        }
      }
      else {
        System.err.println("Function spec must begin with the number of variables "+ ttype + " "+st.sval);
        System.exit(1);
      }
  
      ttype = st.nextToken();
      if (ttype == StreamTokenizer.TT_WORD) {
        if (st.sval.equals("m"))
          min = true;
        else if (st.sval.equals("M"))
          min = false;
        else {
          System.err.println("Function spec must have \"m\" or \"M\" for min or max terms, not "+st.sval);
          System.exit(1);
        }
      }
      else {
        System.err.println("Function spec must have \"m\" or \"M\" for min or max terms");
        System.exit(1);
      }

      // You should never write impenetrable bit twiddling code like the
      // following line.
      int mask = (((1 << 31) >> (numVars - 1))) >>> (32 - numVars);
      int maxTerm = 1 << numVars;
      ttype = st.nextToken();
      while (ttype == StreamTokenizer.TT_NUMBER) {
        int term = (int)st.nval;
        if (!min)
          term = (~term) & mask;
        Integer termInt = new Integer(term);
        if (terms.contains(termInt)) {
          System.err.println("Function spec contains redundant term "+term);
          System.exit(1);
        }
        if ((term >= maxTerm) || (term < 0)) {
          System.err.println("Term "+term+ " is out of range (max="+maxTerm+")");
          System.exit(1);
        }
        terms.add(termInt);
        ttype = st.nextToken();
      }

      if (ttype == StreamTokenizer.TT_WORD) {
        if (!((st.sval.equals("d")) || (st.sval.equals("D")))) {
          System.err.println("Function spec must have \"d\" or \"D\" for don't care terms, not "+st.sval);
          System.exit(1);
        }
      }
      else
        return;

      ttype = st.nextToken();
      while (ttype == StreamTokenizer.TT_NUMBER) {
        int term = (int)st.nval;
        if (!min)
          term = (~term) & mask;
        Integer termInt = new Integer(term);
        if (terms.contains(termInt)) {
          System.err.println("Function spec contains redundant term "+term);
          System.exit(1);
        }
        if ((term >= maxTerm) || (term < 0)) {
          System.err.println("Term "+term+ " is out of range (max="+maxTerm+")");
          System.exit(1);
        }
        dontCares.add(termInt);
        ttype = st.nextToken();
      }
    }
    catch (java.io.IOException e) {
      System.err.println("Failed to read the input for some reason");
      System.exit(1);
    }
  }
}
